% Generate example of IIR and FIR filters
% Show how the impulse response is infinite for IIR but finite for FIR 
% (examples from Taylor 10.4.1-2)

clear
close all
clc

%% IIR filter (Example from Taylor 10.4.1-2)
a = [1 -0.8];
b = 1;

x = zeros(50,1);
x(1) = 1;
y = filter(b,a,x);

figure
stem(y, 'r')
axis tight
xlabel('time'); ylabel('IIR impulse response');

%% Oscillatory IIR filter  (Example from Taylor 10.4.1-2)

% Pure sinusoidal
a = [1 -1.8 1];
b = 1;

x = zeros(50,1);
x(1) = 1;
y = filter(b,a,x);

figure
subplot(121)
stem(y, 'r')
axis tight
xlabel('time'); ylabel('IIR impulse response');

% Sinusoidal decay
a = [1 -1.78 0.9];
b = 1;

x = zeros(50,1);
x(1) = 1;
y = filter(b,a,x);

subplot(122)
stem(y, 'r')
axis tight
xlabel('time'); ylabel('IIR impulse response');


%% FIR filter response to sequence
b = [3 -1 2 1];
a = 1;
x = [0 1 2 1 0 -1 -2 -1 0 0];

y = filter(b,a,x);

figure
subplot(221); stem(x, 'r'); 
axis tight; xlabel('time'); ylabel('input');
subplot(222); stem(y, 'b')
axis tight; xlabel('time'); ylabel('output'); title('filter()') 

% Same FIR, but now calculate its impulse response
xi = zeros(10,1);
xi(1) = 1;
y = filter(b,a,xi);

subplot(223)
stem(y, 'r')
axis tight
xlabel('time'); ylabel('FIR impulse response');

% Now we convolve the impulse response with the original input
% and the result is identical to that of using function filter()

y = conv(x,y);
subplot(224)
stem(y(1:length(x)), 'b')
axis tight; xlabel('time'); ylabel('output'); title('conv()')
axis tight

