% Synthesize speech sample using vocal tract filter in Taylor 10.5.4

clear
close all
clc

%% Generate frequency response
n = 1024; % FFT size
fs = 16e3;

% Pole definitions
p1 = 0.963 + 0.116*j;
p2 = 0.619 + 0.719*j;
p3 = 0.370 + 0.874*j;

% Compute transfer function denominator
a1 = conv([1 -p1],[1 -p1']);
a2 = conv([1 -p2],[1 -p2']);
a3 = conv([1 -p3],[1 -p3']);
a = conv(conv(a1,a2),a3);

% Verify roots of denominator
roots(a)

% Plot frequency response
subplot(311)
[h,w] = freqz(1,a,n,fs);
plot(w,log(abs(h)))
axis tight

%% Synthesize sound
f0 = 75;

% Generate train pulse
x = zeros(fs/f0,1);
x(1) = 1;
x = repmat(x,f0,1);

% Convolve with filter transfer function
y = filter(1,a,x);

subplot(312)
plot(x, 'r')
hold on
plot(y, 'b')

sound(y,fs);

subplot(313)
yf = fft(y, n);
plot(log(abs(yf(1:n/2))))
axis tight